@@ -1,10 +1,10 @@
 ########################################################################
 # Writer.pm - write an XML document.
 # Copyright (c) 1999 by Megginson Technologies.
-# Copyright (c) 2004 by Joseph Walton <joe@kafsemo.org>.
-# No warranty.  Commercial and non-commercial use freely permitted.
-#
-# $Id: Writer.pm,v 1.27 2004/05/25 19:05:38 josephw Exp $
+# Copyright (c) 2003 Ed Avis <ed@membled.com>
+# Copyright (c) 2004-2010 Joseph Walton <joe@kafsemo.org>
+# Redistribution and use in source and compiled forms, with or without
+# modification, are permitted under any circumstances.  No warranty.
 ########################################################################
 
 package XML::Writer;
@@ -15,8 +15,9 @@ use strict;
 use vars qw($VERSION);
 use Carp;
 use IO::Handle;
-$VERSION = "0.510";
+$VERSION = "0.900";
 
+use overload '""' => \&_overload_string;
 
 
 ########################################################################
@@ -40,7 +41,7 @@ sub new {
                                 # from within XML::Writer::Namespaces::new()
   if ($params{NAMESPACES}) {
     delete $params{NAMESPACES};
-    return new XML::Writer::Namespaces(%params);
+    return XML::Writer::Namespaces->new(%params);
   }
 
                                 # Set up $self and basic parameters
@@ -49,7 +50,9 @@ sub new {
   my $unsafe = $params{UNSAFE};
   my $newlines = $params{NEWLINES};
   my $dataMode = $params{DATA_MODE};
-  my $dataIndent = $params{DATA_INDENT} || 0;
+  my $dataIndent;
+  my $selfcontained_output;
+  my $use_selfcontained_output = 0;
 
                                 # If the NEWLINES parameter is specified,
                                 # set the $nl variable appropriately
@@ -58,6 +61,16 @@ sub new {
     $nl = "\n";
   }
 
+  my $outputEncoding = $params{ENCODING} || "";
+  my ($checkUnencodedRepertoire, $escapeEncoding);
+  if (lc($outputEncoding) eq 'us-ascii') {
+    $checkUnencodedRepertoire = \&_croakUnlessASCII;
+    $escapeEncoding = \&_escapeASCII;
+  } else {
+    my $doNothing = sub {};
+    $checkUnencodedRepertoire = $doNothing;
+    $escapeEncoding = $doNothing;
+  }
 
                                 # Parse variables
   my @elementStack = ();
@@ -79,6 +92,10 @@ sub new {
     while ($atts->[$i]) {
       my $aname = $atts->[$i++];
       my $value = _escapeLiteral($atts->[$i++]);
+      $value =~ s/\x0a/\&#10\;/g;
+      $value =~ s/\x0d/\&#13\;/g;
+      $value =~ s/\x09/\&#9\;/g;
+      &{$escapeEncoding}($value);
       $output->print(" $aname=\"$value\"");
     }
   };
@@ -88,6 +105,9 @@ sub new {
                                 # and then call the regular ones.
   my $end = sub {
     $output->print("\n");
+
+    return $selfcontained_output
+        if $use_selfcontained_output and defined wantarray;
   };
 
   my $SAFE_end = sub {
@@ -109,10 +129,12 @@ sub new {
       $standalone = 'yes';
     }
 
-    # This line is questionable, but changing current behaviour
-    # may be a bad idea. There seems to be a mismatch with the
-    # documentation, though.
-    $encoding = "UTF-8" unless $encoding;
+    # Only include an encoding if one has been explicitly supplied,
+    #  either here or on construction. Allow the empty string
+    #  to suppress it.
+    if (!defined($encoding)) {
+      $encoding = $outputEncoding;
+    }
     $output->print("<?xml version=\"1.0\"");
     if ($encoding) {
       $output->print(" encoding=\"$encoding\"");
@@ -150,9 +172,9 @@ sub new {
   my $SAFE_pi = sub {
     my ($name, $data) = (@_);
     $seen{ANYTHING} = 1;
-    if (($name =~ /^xml/i) && ($name !~ /^xml-stylesheet$/i)) {
+    if (($name =~ /^xml/i) && ($name !~ /^xml-(stylesheet|model)$/i)) {
       carp("Processing instruction target begins with 'xml'");
-    } 
+    }
 
     if ($name =~ /\?\>/ || (defined($data) && $data =~ /\?\>/)) {
       croak("Processing instruction may not contain '?>'");
@@ -165,8 +187,14 @@ sub new {
 
   my $comment = sub {
     my $data = $_[0];
+    if ($dataMode && $elementLevel) {
+      $output->print("\n");
+      $output->print($dataIndent x $elementLevel);
+    }
     $output->print("<!-- $data -->");
-    if ($elementLevel == 0) {
+    if ($dataMode && $elementLevel) {
+      $hasElement = 1;
+    } elsif ($elementLevel == 0) {
       $output->print("\n");
       $hasHeading = 1;
     }
@@ -181,6 +209,7 @@ sub new {
     if ($data =~ /-->/) {
       croak("Comment may not contain '-->'");
     } else {
+      &{$checkUnencodedRepertoire}($data);
       $seen{ANYTHING} = 1;
       &{$comment};
     }
@@ -190,11 +219,11 @@ sub new {
     my ($name, $publicId, $systemId) = (@_);
     $output->print("<!DOCTYPE $name");
     if ($publicId) {
-      unless ($systemId) {
+      unless ( defined $systemId) {
         croak("A DOCTYPE declaration with a public ID must also have a system ID");
       }
       $output->print(" PUBLIC \"$publicId\" \"$systemId\"");
-    } elsif ($systemId) {
+    } elsif ( defined $systemId ) {
       $output->print(" SYSTEM \"$systemId\"");
     }
     $output->print(">\n");
@@ -218,7 +247,7 @@ sub new {
     my $name = $_[0];
     if ($dataMode && ($hasHeading || $elementLevel)) {
       $output->print("\n");
-      $output->print(" " x ($elementLevel * $dataIndent));
+      $output->print($dataIndent x $elementLevel);
     }
     $elementLevel++;
     push @elementStack, $name;
@@ -237,7 +266,9 @@ sub new {
   my $SAFE_startTag = sub {
     my $name = $_[0];
 
-    _checkAttributes(\@_);
+    _croakUnlessValidName($name);
+    &{$checkUnencodedRepertoire}($name);
+    _checkAttributes(\@_, $checkUnencodedRepertoire);
 
     if ($seen{ELEMENT} && $elementLevel == 0) {
       croak("Attempt to insert start tag after close of document element");
@@ -258,7 +289,7 @@ sub new {
     my $name = $_[0];
     if ($dataMode && ($hasHeading || $elementLevel)) {
       $output->print("\n");
-      $output->print(" " x ($elementLevel * $dataIndent));
+      $output->print($dataIndent x $elementLevel);
     }
     $output->print("<$name");
     &{$showAttributes}(\@_);
@@ -271,7 +302,9 @@ sub new {
   my $SAFE_emptyTag = sub {
     my $name = $_[0];
 
-    _checkAttributes(\@_);
+    _croakUnlessValidName($name);
+    &{$checkUnencodedRepertoire}($name);
+    _checkAttributes(\@_, $checkUnencodedRepertoire);
 
     if ($seen{ELEMENT} && $elementLevel == 0) {
       croak("Attempt to insert empty tag after close of document element");
@@ -295,7 +328,7 @@ sub new {
     $elementLevel--;
     if ($dataMode && $hasElement) {
       $output->print("\n");
-      $output->print(" " x ($elementLevel * $dataIndent));
+      $output->print($dataIndent x $elementLevel);
     }
     $output->print("</$name$nl>");
     if ($dataMode) {
@@ -323,6 +356,7 @@ sub new {
       $data =~ s/\</\&lt\;/g;
       $data =~ s/\>/\&gt\;/g;
     }
+    &{$escapeEncoding}($data);
     $output->print($data);
     $hasData = 1;
   };
@@ -333,6 +367,7 @@ sub new {
     } elsif ($dataMode && $hasElement) {
       croak("Mixed content not allowed in data mode: characters");
     } else {
+      _croakUnlessDefinedCharacters($_[0]);
       &{$characters};
     }
   };
@@ -361,6 +396,8 @@ sub new {
     } elsif ($dataMode && $hasElement) {
       croak("Mixed content not allowed in data mode: characters");
     } else {
+      _croakUnlessDefinedCharacters($_[0]);
+      &{$checkUnencodedRepertoire}($_[0]);
       &{$cdata};
     }
   };
@@ -416,19 +453,67 @@ sub new {
 
   $self->{'ANCESTOR'} = sub {
     my ($n) = (@_);
-    return $elementStack[$#elementStack-$n];
+    if ($n < scalar(@elementStack)) {
+      return $elementStack[$#elementStack-$n];
+    } else {
+      return undef;
+    }
   };
 
                                 # Set and get the output destination.
   $self->{'GETOUTPUT'} = sub {
-    return $output;
+    if (ref($output) ne 'XML::Writer::_PrintChecker') {
+      return $output;
+    } else {
+      return $output->{HANDLE};
+    }
   };
 
   $self->{'SETOUTPUT'} = sub {
     my $newOutput = $_[0];
+
+    if (defined($newOutput) && !ref($newOutput)) {
+      if ('self' eq $newOutput ) {
+        $newOutput = \$selfcontained_output;
+        $use_selfcontained_output = 1;
+      } else {
+        die "Output must be a handle, a reference or 'self'";
+      }
+    }
+
+    if (ref($newOutput) eq 'SCALAR') {
+      $output = XML::Writer::_String->new($newOutput);
+    } else {
                                 # If there is no OUTPUT parameter,
                                 # use standard output
-    $output = $newOutput || \*STDOUT;
+      $output = $newOutput || \*STDOUT;
+      if ($outputEncoding && (ref($output) eq 'GLOB' || $output->isa('IO::Handle'))) {
+        if (lc($outputEncoding) eq 'utf-8') {
+          binmode($output, ':encoding(utf-8)');
+        } elsif (lc($outputEncoding) eq 'us-ascii') {
+          binmode($output, ':encoding(us-ascii)');
+        } else {
+          die 'The only supported encodings are utf-8 and us-ascii';
+        }
+      }
+    }
+
+    if ($params{CHECK_PRINT}) {
+      $output = XML::Writer::_PrintChecker->new($output);
+    }
+  };
+
+  $self->{OVERLOADSTRING} = sub {
+      # if we don't use the self-contained output,
+      # simple passthrough
+      return $use_selfcontained_output ? $selfcontained_output : undef;
+  };
+
+  $self->{TOSTRING} = sub {
+      die "'to_string' can only be used with self-contained output\n"
+          unless $use_selfcontained_output;
+
+      return $selfcontained_output;
   };
 
   $self->{'SETDATAMODE'} = sub {
@@ -440,13 +525,24 @@ sub new {
   };
 
   $self->{'SETDATAINDENT'} = sub {
-    $dataIndent = $_[0];
+    if ($_[0] =~ /^\s*$/) {
+      $dataIndent = $_[0];
+    } else {
+      $dataIndent = ' ' x $_[0];
+    }
   };
 
   $self->{'GETDATAINDENT'} = sub {
-    return $dataIndent;
+    if ($dataIndent =~ /^ *$/) {
+      return length($dataIndent);
+    } else {
+      return $dataIndent;
+    }
   };
 
+                                # Set the indent.
+  &{$self->{'SETDATAINDENT'}}($params{'DATA_INDENT'} || '');
+
                                 # Set the output.
   &{$self->{'SETOUTPUT'}}($params{'OUTPUT'});
 
@@ -528,8 +624,8 @@ sub endTag {
 # Write a simple data element.
 #
 sub dataElement {
-  my ($self, $name, $data, %atts) = (@_);
-  $self->startTag($name, %atts);
+  my ($self, $name, $data, @atts) = (@_);
+  $self->startTag($name, @atts);
   $self->characters($data);
   $self->endTag($name);
 }
@@ -666,6 +762,12 @@ sub addPrefix {
 sub removePrefix {
 }
 
+sub to_string {
+    my $self = shift;
+
+    $self->{TOSTRING}->();
+}
+
 
 
 ########################################################################
@@ -673,21 +775,27 @@ sub removePrefix {
 ########################################################################
 
 #
-# Private: check for duplicate attributes.
+# Private: check for duplicate attributes and bad characters.
 # Note - this starts at $_[1], because $_[0] is assumed to be an
 # element name.
 #
 sub _checkAttributes {
   my %anames;
   my $i = 1;
+  my $checkUnencodedRepertoire = $_[1];
+
   while ($_[0]->[$i]) {
     my $name = $_[0]->[$i];
-    $i += 2;
+    $i += 1;
     if ($anames{$name}) {
       croak("Two attributes named \"$name\"");
     } else {
       $anames{$name} = 1;
     }
+    _croakUnlessValidName($name);
+    &{$checkUnencodedRepertoire}($name);
+    _croakUnlessDefinedCharacters($_[0]->[$i]);
+    $i += 1;
   }
 }
 
@@ -705,7 +813,52 @@ sub _escapeLiteral {
   return $data;
 }
 
+sub _escapeASCII($) {
+  $_[0] =~ s/([^\x00-\x7F])/sprintf('&#x%X;', ord($1))/ge;
+}
 
+sub _croakUnlessASCII($) {
+  if ($_[0] =~ /[^\x00-\x7F]/) {
+    croak('Non-ASCII characters are not permitted in this part of a US-ASCII document');
+  }
+}
+
+# Enforce XML 1.0, section 2.2's definition of "Char" (only reject low ASCII,
+#  so as not to require Unicode support from perl)
+sub _croakUnlessDefinedCharacters($) {
+  if ($_[0] =~ /([\x00-\x08\x0B-\x0C\x0E-\x1F])/) {
+    croak(sprintf('Code point \u%04X is not a valid character in XML', ord($1)));
+  }
+}
+
+# Ensure element and attribute names are non-empty, contain no whitespace and are
+#  otherwise valid XML names
+sub _croakUnlessValidName($) {
+  if ($_[0] eq '') {
+    croak('Empty identifiers are not permitted in this part of an XML document');
+  }
+  if ($_[0] =~ /\s/) {
+    croak('Space characters are not permitted in this part of an XML identifier');
+  }
+
+  # From REC-xml-20081126
+  #  [4]   	NameStartChar	   ::=   	":" | [A-Z] | "_" | [a-z] | [#xC0-#xD6] | [#xD8-#xF6] | [#xF8-#x2FF] | [#x370-#x37D] | [#x37F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] | [#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] | [#x10000-#xEFFFF]
+  #  [4a]   	NameChar	   ::=   	NameStartChar | "-" | "." | [0-9] | #xB7 | [#x0300-#x036F] | [#x203F-#x2040]
+  #  [5]   	Name	   ::=   	NameStartChar (NameChar)*
+
+  if ($_[0] !~ /^[:A-Z_a-z\x{C0}-\x{D6}\x{D8}-\x{F6}\x{F8}-\x{2FF}\x{370}-\x{37D}\x{37F}-\x{1FFF}\x{200C}-\x{200D}\x{2070}-\x{218F}\x{2C00}-\x{2FEF}\x{3001}-\x{D7FF}\x{F900}-\x{FDCF}\x{FDF0}-\x{FFFD}\x{10000}\x{EFFFF}][-.0-9\x{B7}\x{0300}-\x{036F}\x{203F}-\x{2040}:A-Z_a-z\x{C0}-\x{D6}\x{D8}-\x{F6}\x{F8}-\x{2FF}\x{370}-\x{37D}\x{37F}-\x{1FFF}\x{200C}-\x{200D}\x{2070}-\x{218F}\x{2C00}-\x{2FEF}\x{3001}-\x{D7FF}\x{F900}-\x{FDCF}\x{FDF0}-\x{FFFD}\x{10000}\x{EFFFF}]*$/) {
+    croak('Not a valid XML name: '.$_[0]);
+  }
+
+	  #	  ":" | [A-Z] | "_" | [a-z] | [#xC0-#xD6] | [#xD8-#xF6] | [#xF8-#x2FF] | [#x370-#x37D] | [#x37F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] | [#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] | [#x10000-#xEFFFF]
+
+	  #	  | "-" | "." | [0-9] | #xB7 | [#x0300-#x036F] | [#x203F-#x2040]
+}
+
+sub _overload_string {
+    my $self = shift;
+    $self->{OVERLOADSTRING}->() || overload::StrVal($self);
+}
 
 ########################################################################
 # XML::Writer::Namespaces - subclass for Namespace processing.
@@ -746,7 +899,7 @@ sub new {
   delete $prefixMap{$defaultPrefix} if ($defaultPrefix);
 
                                 # Create an instance of the parent.
-  my $self = new XML::Writer(%params);
+  my $self = XML::Writer->new(%params);
 
                                 # Snarf the parent's methods that we're
                                 # going to override.
@@ -795,9 +948,11 @@ sub new {
       $clashMap{$p} = $u;
     }
 
-    while (!defined($prefix) || ($clashMap{$prefix} && $clashMap{$prefix} ne $uri)) {
-      $prefix = "__NS$prefixCounter";
-      $prefixCounter++;
+    if (!defined($prefix)) {
+      do {
+        $prefix = "__NS$prefixCounter";
+        $prefixCounter++;
+      } while ($clashMap{$prefix});
     }
 
     return $prefix;
@@ -832,23 +987,10 @@ sub new {
         $nsCopyFlag = 1;
       }
       $uriMap->{''} = $uri;
-      
+
                                 # Is there a straight-forward prefix?
     } elsif ($prefix) {
-      unless ($nsDecls->{$uri}) {
-                                # Copy on write (FIXME: duplicated)
-        unless ($nsCopyFlag) {
-          $uriMap = {%{$uriMap}};
-          $nsDecls = {%{$nsDecls}};
-          $nsCopyFlag = 1;
-        }
-        $nsDecls->{$uri} = $prefix;
-        $uriMap->{$prefix} = $uri;
-        push @{$atts}, "xmlns:$prefix";
-        push @{$atts}, $uri;
-      }
       $$nameref = "$prefix:$local";
-
     } else {
       $prefix = &{$genPrefix}($uri);
       unless ($nsCopyFlag) {
@@ -870,22 +1012,31 @@ sub new {
   #
   my $nsProcess = sub {
     if (ref($_[0]->[0]) eq 'ARRAY') {
-      &{$processName}(\$_[0]->[0], $_[0], 0);
+      my $x = \@{$_[0]->[0]};
+      &{$processName}(\$x, $_[0], 0);
+      splice(@{$_[0]}, 0, 1, $x);
     }
     my $i = 1;
     while ($_[0]->[$i]) {
       if (ref($_[0]->[$i]) eq 'ARRAY') {
-        &{$processName}(\$_[0]->[$i], $_[0], 1);
+        my $x = \@{$_[0]->[$i]};
+        &{$processName}(\$x, $_[0], 1);
+        splice(@{$_[0]}, $i, 1, $x);
       }
       $i += 2;
     }
 
-    # We only do this for the outermost element
+    # We do this if any declarations are forced, due either to
+    #  constructor arguments or to a call during processing.
     if (@forcedNSDecls) {
       foreach (@forcedNSDecls) {
         my @dummy = ($_, 'dummy');
         my $d2 = \@dummy;
-        &{$processName}(\$d2, $_[0], 1);
+        if ($defaultPrefix && ($_ eq $defaultPrefix)) {
+          &{$processName}(\$d2, $_[0], 0);
+        } else {
+          &{$processName}(\$d2, $_[0], 1);
+        }
       }
       @forcedNSDecls = ();
     }
@@ -1060,6 +1211,47 @@ sub forceNSDecl
 }
 
 
+package XML::Writer::_String;
+
+# Internal class, behaving sufficiently like an IO::Handle,
+#  that stores written output in a string
+#
+# Heavily inspired by Simon Oliver's XML::Writer::String
+
+sub new
+{
+  my $class = shift;
+  my $scalar_ref = shift;
+  return bless($scalar_ref, $class);
+}
+
+sub print
+{
+  ${(shift)} .= join('', @_);
+  return 1;
+}
+
+
+package XML::Writer::_PrintChecker;
+
+use Carp;
+
+sub new
+{
+  my $class = shift;
+  return bless({HANDLE => shift}, $class);
+}
+
+sub print
+{
+  my $self = shift;
+  if ($self->{HANDLE}->print(shift)) {
+    return 1;
+  } else {
+    croak "Failed to write output: $!";
+  }
+}
+
 1;
 __END__
 
@@ -1076,10 +1268,10 @@ XML::Writer - Perl extension for writing XML documents.
   use XML::Writer;
   use IO::File;
 
-  my $output = new IO::File(">output.xml");
+  my $output = IO::File->new(">output.xml");
 
-  my $writer = new XML::Writer(OUTPUT => $output);
-  $writer->startTag("greeting", 
+  my $writer = XML::Writer->new(OUTPUT => $output);
+  $writer->startTag("greeting",
                     "class" => "simple");
   $writer->characters("Hello, world!");
   $writer->endTag("greeting");
@@ -1119,7 +1311,7 @@ elements can optionally be indented based as their nesting level.
 
 Create a new XML::Writer object:
 
-  my $writer = new XML::Writer(OUTPUT => $output, NEWLINES => 1);
+  my $writer = XML::Writer->new(OUTPUT => $output, NEWLINES => 1);
 
 Arguments are an anonymous hash array of parameters:
 
@@ -1127,9 +1319,22 @@ Arguments are an anonymous hash array of parameters:
 
 =item OUTPUT
 
-An object blessed into IO::Handle or one of its subclasses (such as
-IO::File); if this parameter is not present, the module will write to
-standard output.
+An object blessed into IO::Handle or one of its subclasses (such as IO::File),
+or a reference to a string, or any blessed object that has a print() method;
+if this parameter is not present, the module will write to standard output. If
+a string reference is passed, it will capture the generated XML (as a string;
+to get bytes use the C<Encode> module).
+
+If the string I<self> is passed, the output will be captured internally by the
+object, and can be accessed via the C<to_string()> method, or by calling the
+object in a string context.
+
+    my $writer = XML::Writer->new( OUTPUT => 'self' );
+
+    $writer->dataElement( hello => 'world' );
+
+    print $writer->to_string;  # outputs <hello>world</hello>
+    print "$writer";           # ditto
 
 =item NAMESPACES
 
@@ -1139,7 +1344,7 @@ reference in the place of element and attribute names, as in the
 following example:
 
   my $rdfns = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
-  my $writer = new XML::Writer(NAMESPACES => 1);
+  my $writer = XML::Writer->new(NAMESPACES => 1);
   $writer->startTag([$rdfns, "Description"]);
 
 The first member of the array is a namespace URI, and the second part
@@ -1156,7 +1361,7 @@ namespace URIs:
 
 
   my $rdfns = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
-  my $writer = new XML::Writer(NAMESPACES => 1,
+  my $writer = XML::Writer->new(NAMESPACES => 1,
                                PREFIX_MAP => {$rdfns => 'rdf'});
 
 The keys in the hash table are namespace URIs, and the values are the
@@ -1179,7 +1384,7 @@ ancestor from the same namespace.
 A true or false value; if this parameter is present and its value is
 true, then the module will insert an extra newline before the closing
 delimiter of start, end, and empty tags to guarantee that the document
-does not end up as a single, long line.  If the paramter is not
+does not end up as a single, long line.  If the parameter is not
 present, the module will not insert the newlines.
 
 =item UNSAFE
@@ -1200,9 +1405,25 @@ elements as content.
 
 =item DATA_INDENT
 
-A numeric value; if this parameter is present, it represents the
+A numeric value or white space; if this parameter is present, it represents the
 indent step for elements in data mode (it will be ignored when not in
-data mode).
+data mode). If it is white space it will be repeated for each level of
+indentation.
+
+=item ENCODING
+
+A character encoding to use for the output; currently this must be one of
+'utf-8' or 'us-ascii'.
+If present, it will be used for the underlying character encoding and as the
+default in the XML declaration.
+All character data should be passed as Unicode strings when an encoding is
+set.
+
+=item CHECK_PRINT
+
+A true or false value; if this parameter is present and its value is
+true, all prints to the underlying output will be checked for success. Failures
+will cause a croak rather than being ignored.
 
 =back
 
@@ -1214,13 +1435,17 @@ closed:
 
   $writer->end();
 
+If I<OUTPUT> as been set to I<self>, C<end()> will return the generated
+document as well.
+
 =item xmlDecl([$encoding, $standalone])
 
 Add an XML declaration to the beginning of an XML document.  The
 version will always be "1.0".  If you provide a non-null encoding or
-standalone argument, its value will appear in the declaration (and
+standalone argument, its value will appear in the declaration (any
 non-null value for standalone except 'no' will automatically be
-converted to 'yes').
+converted to 'yes'). If not given here, the encoding will be taken from the
+ENCODING argument. Pass the empty string to suppress this behaviour.
 
   $writer->xmlDecl("UTF-8");
 
@@ -1238,7 +1463,7 @@ just a system ID by passing 'undef' for the publicId.
 Add a comment to an XML document.  If the comment appears outside the
 document element (either before the first start tag or after the last
 end tag), the module will add a carriage return after it to improve
-readability:
+readability. In data mode, comments will be treated as empty tags:
 
   $writer->comment("This is a comment");
 
@@ -1515,12 +1740,37 @@ This error reporting can catch many hidden bugs in Perl programs that
 create XML documents; however, if necessary, it can be turned off by
 providing an UNSAFE parameter:
 
-  my $writer = new XML::Writer(OUTPUT => $output, UNSAFE => 1);
+  my $writer = XML::Writer->new(OUTPUT => $output, UNSAFE => 1);
 
+=head2 PRINTING OUTPUT
+
+If I<OUTPUT> has been set to I<self> and the object has been called in
+a string context, it'll return the xml document.
+
+=over 4
+
+=item to_string
+
+If I<OUTPUT> has been set to I<self>, calls an implicit C<end()> on the
+document and prints it. Dies if I<OUTPUT> has been set to anything else.
+
+=back
 
 =head1 AUTHOR
 
-David Megginson, david@megginson.com
+David Megginson E<lt>david@megginson.comE<gt>
+
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (c) 1999 by Megginson Technologies.
+
+Copyright (c) 2003 Ed Avis E<lt>ed@membled.comE<gt>
+
+Copyright (c) 2004-2010 Joseph Walton E<lt>joe@kafsemo.orgE<gt>
+
+Redistribution and use in source and compiled forms, with or without
+modification, are permitted under any circumstances.  No warranty.
 
 =head1 SEE ALSO
 
